package blink.events;

#if flash
typedef EventDispatcher = flash.events.EventDispatcher;
#else

import blink.events.IEventDispatcher;

class Listener
{
   public var mListner : Function;
   public var mUseCapture : Bool;
   public var mPriority : Int;

   public function new(inListener,inUseCapture,inPriority)
   {
      mListner = inListener;
      mUseCapture = inUseCapture;
      mPriority = inPriority;
   }

   public function Is(inListener,inCapture)
   {
      return mListner==inListener && mUseCapture == inCapture;
   }

   public function dispatchEvent(event : Event)
   {
      mListner(event);
   }
}

typedef ListenerList = Array<Listener>;

typedef EventMap = Hash<ListenerList>;

class EventDispatcher implements IEventDispatcher
{
   var mTarget:IEventDispatcher;
   var mEventMap : EventMap;

   public function new(?target : IEventDispatcher) : Void
   {
      mTarget =target;
      mEventMap = new EventMap();
   }

   public function addEventListener(type:String, inListener:Function,
       ?useCapture:Bool /*= false*/, ?inPriority:Int /*= 0*/,
       ?useWeakReference:Bool /*= false*/):Void
   {
      var capture:Bool = useCapture==null ? false : useCapture;
      var priority:Int = inPriority==null ? 0 : inPriority;

      var list = mEventMap.get(type);
      if (list==null)
      {
         list = new ListenerList();
         mEventMap.set(type,list);
      }

      list.push( new Listener(inListener,capture,priority) );
   }

   public function dispatchEvent(event : Event) : Bool
   {
      var list = mEventMap.get(event.type);
      if (list!=null)
      {
         for(i in 0...list.length)
            list[i].dispatchEvent(event);
         return true;
      }

      return false;
   }

   public function hasEventListener(type : String)
   {
      return mEventMap.exists(type);
   }
   public function removeEventListener(type : String, listener : Function,
              ?inCapture : Bool) : Void
   {
      if (!mEventMap.exists(type)) return;

      var list = mEventMap.get(type);
      var capture:Bool = inCapture==null ? false : inCapture;
      for(i in 0...list.length)
      {
         if (list[i].Is(listener,capture))
         {
             list.splice(i,1);
             return;
         }
      }
   }
   public function willTrigger(type : String) : Bool
   {
      return hasEventListener(type);
   }


}



#end
