package ui.containers;

import ui.core.Container;
import ui.core.Element;
import ui.core.Rectangle;
import ui.core.Size;
import ui.controls.Label;
import ui.core.Layout;

class FieldLayout extends Container {
  public var label(default, null) : Label;
  public var element(default, setElement) : Element;
  public var elwidth : Float;
  public var labelposition : LabelPosition;
  
	public function new(elwidth : Float, ?text : String, ?el : Element, ?labelposition : LabelPosition) {
    super();
    vLayout = AnchorStart(0);
		label = new Label(text);
    var l : Element = label;
    l.parent = this;
    element = el;
    this.elwidth = elwidth;
    this.labelposition = if(labelposition == null) Left else labelposition;
	}
  
  public function setElement(?el : Element) {
    element = el;
    if(el != null) {
      el.parent = this;
      el.hLayout = Margins(0);
    }
    return el;
  }
  
  public override function render() {
    super.render();
    label.render();
    if(element != null) element.render();
  }
  
  public override function calculateBounds(e : Element) : Rectangle<Float> {
    return if(e == label)
      calcLabelBounds();
    else
      calcElementBounds();
  }
  
  public function calcLabelBounds() : Rectangle<Float> {
    var f = label.fitSize();
    var r = bounds();
    label.maxSize.width = r.width-elwidth;
    var h = boundedSide(Margins(0), r.width-elwidth, label.minSize.width, label.maxSize.width, f.width); 
    var v = boundedSide(label.vLayout, r.height, label.minSize.height, label.maxSize.height, f.height); 
    switch(labelposition) {
      case Left:
        return { x : h.d, y : v.d, width : h.l, height : v.l };
      case Right:
        return { x : h.d+elwidth, y : v.d, width : h.l, height : v.l };
    }
  }
  
  public function calcElementBounds() : Rectangle<Float> {
    var f = element.fitSize();
    var r = bounds();
    var h = boundedSide(element.hLayout, elwidth, element.minSize.width, element.maxSize.width, f.width); 
    var v = boundedSide(element.vLayout, r.height, element.minSize.height, element.maxSize.height, f.height); 
    switch(labelposition) {
      case Left:
        return { x : h.d+r.width-elwidth, y : v.d, width : h.l, height : v.l };
      case Right:
        return { x : h.d, y : v.d, width : h.l, height : v.l };
    }
  }
  
  public override function fitSize() : Size<Float> {
    var f1 = label.fitSize();
    var f2 = if(element != null)
      element.fitSize();
    else
      { width : 0.0, height : 0.0 };
    return { width : f1.width+f2.width, height : Math.max(f1.height, f2.height) };
  }
}

enum LabelPosition {
  Left;
  Right;
}