package ui.containers;

import ui.core.Element;
import ui.core.Rectangle;
import ui.core.Container;
import ui.core.Size;

class GridLayout extends Container {
  public var rows(default, null) : Int;
  public var cols(default, null) : Int;
  private var grid : Array<Array<Element>>;
  
  public function new(?rows : Int, ?cols : Int) {
    super();
    if(rows == null) rows = 1;
    if(cols == null) cols = 1;
    grid = new Array();
    resize(rows, cols);
  }
  
  public function resize(rows : Int, cols : Int) {
    if(rows < 1 || cols < 1) throw "InvalidNumberOfItems";
    var n = new Array<Array<Element>>();
    var minrows = Math.floor(Math.min(this.rows, rows));
    var mincols = Math.floor(Math.min(this.cols, cols));
    for(i in 0...minrows)
      for(j in 0...mincols)
        n[i][j] = grid[i][j];
    grid = n;
    this.rows = rows;
    this.cols = cols;
  }
  
  public function setElement(el : Element, row : Int, col : Int) {
    if(row >= rows || row < 0 || col >= cols || col < 0) throw "InvalidPosition";
    if(grid[row] == null) grid[row] = new Array();
    grid[row][col] = el;
    el.parent = this;
  }
  
  public function removeElement(row : Int, col : Int) {
    if(row >= rows || row < 0 || col >= cols || col < 0) throw "InvalidPosition";
    grid[row][col].parent = null;
    grid[row][col] = null;
  }
  
  public function getPosition(el : Element) : { row : Int, col : Int } {
    for(i in 0...rows)
      for(j in 0...cols)
        if(el == grid[i][j])
          return { row : i, col : j };
    return null;
  }
  
  public override function calculateBounds(e : Element) : Rectangle<Float> {
    var p = getPosition(e);
    if(p == null) return null;
    var f = e.fitSize();
    var r = bounds();
    var rw = (r.width-spacing*(cols-1))/cols;
    var rh = (r.height-spacing*(rows-1))/rows;
    var h = boundedSide(e.hLayout, rw, e.minSize.width, e.maxSize.width, f.width);
    var v = boundedSide(e.vLayout, rh, e.minSize.height, e.maxSize.height, f.height);
    var x = p.col * (rw+spacing);
    var y = p.row * (rh+spacing);
    return { x : x+h.d, y : y+v.d, width : h.l, height : v.l };
  }
  
  public override function render() {
    super.render();
    for(i in 0...rows)
      for(j in 0...cols)
        if(grid[i] != null && grid[i][j] != null)
          grid[i][j].render();
  }
}