package ui.core;

class Element { 
  public var vLayout : Layout;
  public var hLayout : Layout;
  public var minSize : Size<Float>;
  public var maxSize : Size<Float>;
  public var mouse : MouseInteraction;
  public var key : KeyInteraction;    
  public var context(getContext, null) : IContext;  
  private var _context : IContext;
  
	private function new() {
		vLayout = hLayout = Layout.Center();
    minSize = { width : 0.0, height : 0.0 };
    maxSize = { width : Math.POSITIVE_INFINITY, height : Math.POSITIVE_INFINITY };
    key = new KeyInteraction();
    mouse = new MouseInteraction();
	}
  
  public var parent(default, null) : Container; 
  
  public function bounds() : Rectangle<Float> {
    if(parent == null)
      return null;
    return parent.calculateBounds(this);
  }
  
  public function fitSize() : Size<Float> {
    var r = getContext();
    return if(r==null)
      { width: 0.0, height : 0.0 };
    else
      r.fitSize(); 
  }
  
  public function getView() : IView {
    var p = this;
    while(p.parent != null)
      p = p.parent;
    if(Std.is(p, IView))
      return cast(p, IView);
    else
      return null;
  }
  
  private function getContext() : IContext {
    if(_context == null) {
      var view = getView();
      if(view == null)
        return null;
      var contextclass;
      var cl : Class<Dynamic> = Type.getClass(this);
      while(cl != null) {
        var thiscname = Type.getClassName(cl).split('.').pop();
        var contextcname = view.packagename + "." + thiscname + "Context";
        contextclass = Type.resolveClass(contextcname);
        if(contextclass != null)
          break;
        cl = Type.getSuperClass(cl);
      }
      if(contextclass == null)
        return null;
      _context = Type.createInstance(contextclass, [view, this]);
      KeyInteraction.replace(this, _context.key);
      MouseInteraction.replace(this, _context.mouse);
    }
    return _context;
  }
  
  public function render() {
    var r = getContext();
    if(r!=null)
      r.render(bounds());
  }
}
