package ui.flash9;

import ui.controls.TextElement;
import ui.core.IView;
import ui.core.Size;
import flash.text.TextField;

class TextElementContext<T : TextElement> extends ElementContext<T> {
  public var textfield : TextField;  
  private var _metric : Size<Float>;  
  private var metric(getMetric, null) : Size<Float>;
  private var style : TextElementStyle;
  
  public function new(view : IView, el : T) {
    super(view, el);
    textfield = new TextField();
    sprite.addChild(textfield);
    textfield.multiline = false;    
    textfield.autoSize = flash.text.TextFieldAutoSize.NONE;   
	}
  
  private function getMetric() {
    if(_metric == null || textfield.text != el.text || textfield.defaultTextFormat != style.textformat) {
      textfield.defaultTextFormat = style.textformat;
      textfield.text = el.text;
      var m = textfield.getLineMetrics(0);
      _metric = { width : m.width+4, height : m.height+4 };
    }
    return _metric;
  }  
  
	private override function draw(r : Size<Float>) {
    var g = sprite.graphics;
    g.clear();   
    sprite.filters = style.filters;
    if(style.bg != null) {
      var matr = new flash.geom.Matrix();
      var b = style.bg;
      matr.createGradientBox(r.width, r.height, b.angle, r.width*b.x, r.height*b.y);
      g.beginGradientFill(b.gradientType, b.colors, b.alphas, b.ratios, matr, b.spreadMethod);
    }
    var bw = if(style.border != null) {
      g.lineStyle(style.border.width, style.border.color, style.border.alpha, true);
      style.border.width;
    } else {
      g.lineStyle();
      0.0;
    }    
    if(style.border != null || style.bg != null) {
      if(style.corners == null)
        g.drawRect(bw/2, bw/2, r.width-bw, r.height-bw);
      else {
        var c = style.corners;
        g.drawRoundRectComplex(bw/2, bw/2, r.width-bw, r.height-bw, c.tl, c.tr, c.bl, c.br);
      } 
    }
    if(style.bg != null)
      g.endFill();
    textfield.width = r.width-(bw+style.padding.width)*2;
    textfield.height = metric.height;
    textfield.x = Math.round(bw+style.padding.width);
    textfield.y = Math.round((r.height - metric.height)/2);
  }
  public override function fitSize() : Size<Float> {
    var b = if(style.border == null) 0 else style.border.width*2;
    return { width : metric.width + style.padding.width*2+b, height : metric.height + style.padding.height*2+b };
  }
}

typedef TextElementStyle = {
  textformat : flash.text.TextFormat, 
  padding : { width : Int, height : Int }, 
  filters : Array<Dynamic>, 
  corners : { tr : Int, tl : Int, br : Int, bl : Int }, 
  border :  { width : Int, color : Int , alpha : Float }, 
  bg : { 
    ratios : Array<Int>,
    gradientType : flash.display.GradientType, 
    colors : Array<Int>, 
    angle : Float, 
    alphas : Array<Int> ,
    x : Float,
    y : Float,
    spreadMethod : flash.display.SpreadMethod  }
}